@@ -17,16 +17,19 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
 #include <sys/types.h>
 
-#include <qcursor.h>
-#include <qfile.h>
-#include <qapplication.h>
+#include <QCursor>
+#include <QFile>
+#include <QApplication>
+#include <QByteArray>
+#include <QPixmap>
+#include <QVector>
+#include <QTextStream>
 
-#include "config.h"
 #include "chm_lib.h"
 #include "bitfiddle.h"
 #include "libchmfile.h"
@@ -53,8 +56,10 @@ class KCHMShowWaitCursor
 
 LCHMFileImpl::LCHMFileImpl( )
 {
+	m_envOptions = getenv("KCHMVIEWEROPTS");
 	m_chmFile = NULL;
-	m_home = m_filename = m_home = m_topicsFile = m_indexFile = m_font = QString::null;
+	m_filename = m_font = QString::null;
+	
 	m_entityDecodeMap.clear();
 	m_textCodec = 0;
 	m_textCodecForSpecialFiles = 0;
@@ -117,6 +122,24 @@ bool LCHMFileImpl::loadFile( const QString & archiveName )
 	else
 		m_searchAvailable = false;
 	
+	// Some CHM files have toc and index files, but do not set the name properly.
+	// Some heuristics here.
+	if ( m_topicsFile.isEmpty() && hasFile( "/toc.hhc" ) )
+		m_topicsFile = "/toc.hhc";
+	
+	if ( m_indexFile.isEmpty() && hasFile( "/index.hhk" ) )
+		m_indexFile = "/index.hhk";
+
+	if ( !m_topicsFile.isEmpty() || ( m_lookupTablesValid && hasFile( "/#TOCIDX" ) ) )
+		m_tocAvailable = true;
+	else
+		m_tocAvailable = false;
+		
+	if ( !m_indexFile.isEmpty() || ( m_lookupTablesValid && hasFile( "/$WWKeywordLinks/BTree" ) ) )
+		m_indexAvailable = true;
+	else
+		m_indexAvailable = false;
+	
 	return true;
 }
 
@@ -129,7 +152,12 @@ void LCHMFileImpl::closeAll( )
 	chm_close( m_chmFile );
 	
 	m_chmFile = NULL;
-	m_home = m_filename = m_home = m_topicsFile = m_indexFile = m_font = QString::null;
+	m_filename = m_font = QString::null;
+	
+	m_home.clear();
+	m_topicsFile.clear();
+	m_indexFile.clear();
+	
 	m_entityDecodeMap.clear();
 	m_textCodec = 0;
 	m_textCodecForSpecialFiles = 0;
@@ -310,7 +338,7 @@ QString LCHMFileImpl::decodeEntity( const QString & entity )
 						
 		if ( !valid )
 		{
-			qWarning ( "LCHMFileImpl::decodeEntity: could not decode HTML entity '%s'", entity.ascii() );
+			qWarning ( "LCHMFileImpl::decodeEntity: could not decode HTML entity '%s'", qPrintable( entity ) );
 			return QString::null;
 		}
 
@@ -322,7 +350,7 @@ QString LCHMFileImpl::decodeEntity( const QString & entity )
 
 		if ( it == m_entityDecodeMap.end() )
 		{
-			qWarning ("LCHMFileImpl::decodeEntity: could not decode HTML entity '%s'", entity.ascii());
+			qWarning ("LCHMFileImpl::decodeEntity: could not decode HTML entity '%s'", qPrintable( entity ));
 			return QString::null;
 		}
 		
@@ -333,15 +361,15 @@ QString LCHMFileImpl::decodeEntity( const QString & entity )
 
 inline int LCHMFileImpl::findStringInQuotes (const QString& tag, int offset, QString& value, bool firstquote, bool decodeentities)
 {
-	int qbegin = tag.find ('"', offset);
+	int qbegin = tag.indexOf ('"', offset);
 	
 	if ( qbegin == -1 )
-		qFatal ("LCHMFileImpl::findStringInQuotes: cannot find first quote in <param> tag: '%s'", tag.ascii());
+		qFatal ("LCHMFileImpl::findStringInQuotes: cannot find first quote in <param> tag: '%s'", qPrintable( tag ));
 
-	int qend = firstquote ? tag.find ('"', qbegin + 1) : tag.findRev ('"');
+	int qend = firstquote ? tag.indexOf ('"', qbegin + 1) : tag.lastIndexOf ('"');
 
 	if ( qend == -1 || qend <= qbegin )
-		qFatal ("LCHMFileImpl::findStringInQuotes: cannot find last quote in <param> tag: '%s'", tag.ascii());
+		qFatal ("LCHMFileImpl::findStringInQuotes: cannot find last quote in <param> tag: '%s'", qPrintable( tag ));
 
 	// If we do not need to decode HTML entities, just return.
 	if ( decodeentities )
@@ -416,20 +444,20 @@ bool LCHMFileImpl::searchWord (const QString& text,
 	}
 
 	unsigned char* cursor32 = header + 0x14;
-	u_int32_t node_offset = UINT32ARRAY(cursor32);
+	unsigned int node_offset = UINT32ARRAY(cursor32);
 
 	cursor32 = header + 0x2e;
-	u_int32_t node_len = UINT32ARRAY(cursor32);
+	unsigned int node_len = UINT32ARRAY(cursor32);
 
 	unsigned char* cursor16 = header + 0x18;
-	u_int16_t tree_depth = UINT16ARRAY(cursor16);
+	unsigned short tree_depth = UINT16ARRAY(cursor16);
 
 	unsigned char word_len, pos;
 	QString word;
-	u_int32_t i = sizeof(u_int16_t);
-	u_int16_t free_space;
+	unsigned int i = sizeof(unsigned short);
+	unsigned short free_space;
 
-	QMemArray<unsigned char> buffer(node_len);
+	QVector<unsigned char> buffer(node_len);
 
 	node_offset = GetLeafNodeOffset (searchword, node_offset, node_len, tree_depth);
 
@@ -445,9 +473,9 @@ bool LCHMFileImpl::searchWord (const QString& text,
 		cursor16 = buffer.data() + 6;
 		free_space = UINT16ARRAY(cursor16);
 
-		i = sizeof(u_int32_t) + sizeof(u_int16_t) + sizeof(u_int16_t);
+		i = sizeof(unsigned int) + sizeof(unsigned short) + sizeof(unsigned short);
 		u_int64_t wlc_count, wlc_size;
-		u_int32_t wlc_offset;
+		unsigned int wlc_offset;
 
 		while (i < node_len - free_space)
 		{
@@ -461,7 +489,7 @@ bool LCHMFileImpl::searchWord (const QString& text,
 			if ( pos == 0 )
 				word = wrd_buf;
 			else
-				word = word.mid (0, pos) + wrd_buf;
+				word = word.mid (0, pos) + (const char*) wrd_buf;
 
 			delete[] wrd_buf;
 
@@ -475,7 +503,7 @@ bool LCHMFileImpl::searchWord (const QString& text,
 			cursor32 = buffer.data() + i;
 			wlc_offset = UINT32ARRAY(cursor32);
 
-			i += sizeof(u_int32_t) + sizeof(u_int16_t);
+			i += sizeof(unsigned int) + sizeof(unsigned short);
 			wlc_size =  be_encint (buffer.data() + i, encsz);
 			i += encsz;
 
@@ -519,7 +547,17 @@ bool LCHMFileImpl::searchWord (const QString& text,
 bool LCHMFileImpl::ResolveObject(const QString& fileName, chmUnitInfo *ui) const
 {
 	return m_chmFile != NULL 
-			&& ::chm_resolve_object(m_chmFile, fileName.ascii(), ui) ==
+			&& ::chm_resolve_object(m_chmFile, qPrintable( fileName ), ui) ==
+			CHM_RESOLVE_SUCCESS;
+}
+
+
+bool LCHMFileImpl::hasFile(const QString & fileName) const
+{
+	chmUnitInfo ui;
+	
+	return m_chmFile != NULL
+			&& ::chm_resolve_object(m_chmFile, qPrintable( fileName ), &ui) ==
 			CHM_RESOLVE_SUCCESS;
 }
 
@@ -527,26 +565,21 @@ bool LCHMFileImpl::ResolveObject(const QString& fileName, chmUnitInfo *ui) const
 size_t LCHMFileImpl::RetrieveObject(const chmUnitInfo *ui, unsigned char *buffer,
 								LONGUINT64 fileOffset, LONGINT64 bufferSize) const
 {
-#if USE_BUILTIN_CHMLIB
-	return ::chm_retrieve_object(m_chmFile, ui, buffer, 
-								 fileOffset, bufferSize);
-#else
 	return ::chm_retrieve_object(m_chmFile, const_cast<chmUnitInfo*>(ui),
 								 buffer, fileOffset, bufferSize);
-#endif
 }
 
 
-inline u_int32_t LCHMFileImpl::GetLeafNodeOffset(const QString& text,
-											 u_int32_t initialOffset,
-			u_int32_t buffSize,
-   u_int16_t treeDepth)
+inline unsigned int LCHMFileImpl::GetLeafNodeOffset(const QString& text,
+											 unsigned int initialOffset,
+			unsigned int buffSize,
+   unsigned short treeDepth)
 {
-	u_int32_t test_offset = 0;
+	unsigned int test_offset = 0;
 	unsigned char* cursor16, *cursor32;
 	unsigned char word_len, pos;
-	u_int32_t i = sizeof(u_int16_t);
-	QMemArray<unsigned char> buffer(buffSize);
+	unsigned int i = sizeof(unsigned short);
+	QVector<unsigned char> buffer(buffSize);
 	QString word;
 	
 	while(--treeDepth)
@@ -559,7 +592,7 @@ inline u_int32_t LCHMFileImpl::GetLeafNodeOffset(const QString& text,
 			return 0;
 
 		cursor16 = buffer.data();
-		u_int16_t free_space = UINT16ARRAY(cursor16);
+		unsigned short free_space = UINT16ARRAY(cursor16);
 
 		while (i < buffSize - free_space )
 		{
@@ -573,7 +606,7 @@ inline u_int32_t LCHMFileImpl::GetLeafNodeOffset(const QString& text,
 			if ( pos == 0 )
 				word = wrd_buf;
 			else
-				word = word.mid(0, pos) + wrd_buf;
+				word = word.mid(0, pos) + (const char*) wrd_buf;
 
 			delete[] wrd_buf;
 
@@ -585,7 +618,7 @@ inline u_int32_t LCHMFileImpl::GetLeafNodeOffset(const QString& text,
 			}
 
 			i += word_len + sizeof(unsigned char) +
-					sizeof(u_int32_t) + sizeof(u_int16_t);
+					sizeof(unsigned int) + sizeof(unsigned short);
 		}
 	}
 
@@ -597,7 +630,7 @@ inline u_int32_t LCHMFileImpl::GetLeafNodeOffset(const QString& text,
 
 
 inline bool LCHMFileImpl::ProcessWLC (u_int64_t wlc_count, u_int64_t wlc_size,
-								    u_int32_t wlc_offset, unsigned char ds,
+								    unsigned int wlc_offset, unsigned char ds,
 		  							unsigned char dr, unsigned char cs,
 									unsigned char cr, unsigned char ls,
  									unsigned char lr,
@@ -607,7 +640,7 @@ inline bool LCHMFileImpl::ProcessWLC (u_int64_t wlc_count, u_int64_t wlc_size,
 	int wlc_bit = 7;
 	u_int64_t index = 0, count;
 	size_t length, off = 0;
-	QMemArray<unsigned char> buffer (wlc_size);
+	QVector<unsigned char> buffer (wlc_size);
 	unsigned char *cursor32;
 
 	unsigned char entry[TOPICS_ENTRY_LEN];
@@ -679,10 +712,10 @@ bool LCHMFileImpl::getInfoFromWindows()
 		if ( !RetrieveObject(&ui, buffer, 0, WIN_HEADER_LEN) )
 			return false;
 
-		u_int32_t entries = get_int32_le( (u_int32_t *)(buffer) );
-		u_int32_t entry_size = get_int32_le( (u_int32_t *)(buffer + 0x04) );
+		unsigned int entries = get_int32_le( (unsigned int *)(buffer) );
+		unsigned int entry_size = get_int32_le( (unsigned int *)(buffer + 0x04) );
 		
-		QByteArray uptr(entries * entry_size);
+		QVector<unsigned char> uptr(entries * entry_size);
 		unsigned char* raw = (unsigned char*) uptr.data();
 		
 		if ( !RetrieveObject (&ui, raw, 8, entries * entry_size) )
@@ -691,14 +724,14 @@ bool LCHMFileImpl::getInfoFromWindows()
 		if( !ResolveObject ("/#STRINGS", &ui) )
 			return false;
 
-		for ( u_int32_t i = 0; i < entries; ++i )
+		for ( unsigned int i = 0; i < entries; ++i )
 		{
-			u_int32_t offset = i * entry_size;
+			unsigned int offset = i * entry_size;
 			
-			u_int32_t off_title = get_int32_le( (u_int32_t *)(raw + offset + 0x14) );
-			u_int32_t off_home = get_int32_le( (u_int32_t *)(raw + offset + 0x68) );
-			u_int32_t off_hhc = get_int32_le( (u_int32_t *)(raw + offset + 0x60) );
-			u_int32_t off_hhk = get_int32_le( (u_int32_t *)(raw + offset + 0x64) );
+			unsigned int off_title = get_int32_le( (unsigned int *)(raw + offset + 0x14) );
+			unsigned int off_home = get_int32_le( (unsigned int *)(raw + offset + 0x68) );
+			unsigned int off_hhc = get_int32_le( (unsigned int *)(raw + offset + 0x60) );
+			unsigned int off_hhk = get_int32_le( (unsigned int *)(raw + offset + 0x64) );
 
 			factor = off_title / 4096;
 
@@ -706,7 +739,7 @@ bool LCHMFileImpl::getInfoFromWindows()
 				size = RetrieveObject(&ui, buffer, factor * 4096, BUF_SIZE);
 
 			if ( size && off_title )
-				m_title = QString ((const char*) (buffer + off_title % 4096));
+				m_title = QByteArray( (const char*) (buffer + off_title % 4096) );
 
 			if ( factor != off_home / 4096)
 			{
@@ -715,7 +748,7 @@ bool LCHMFileImpl::getInfoFromWindows()
 			}
 			
 			if ( size && off_home )
-				m_home = QString("/") + QString( (const char*) buffer + off_home % 4096);
+				m_home = QByteArray("/") + QByteArray( (const char*) buffer + off_home % 4096);
 
 			if ( factor != off_hhc / 4096)
 			{
@@ -724,7 +757,7 @@ bool LCHMFileImpl::getInfoFromWindows()
 			}
 		
 			if ( size && off_hhc )
-				m_topicsFile = QString("/") + QString ((const char*) buffer + off_hhc % 4096);
+				m_topicsFile = QByteArray("/") + QByteArray((const char*) buffer + off_hhc % 4096);
 
 			if ( factor != off_hhk / 4096)
 			{
@@ -733,7 +766,7 @@ bool LCHMFileImpl::getInfoFromWindows()
 			}
 
 			if ( size && off_hhk )
-				m_indexFile = QString("/") + QString((const char*) buffer + off_hhk % 4096);
+				m_indexFile = QByteArray("/") + QByteArray((const char*) buffer + off_hhk % 4096);
 		}
 	}
 	return true;
@@ -748,7 +781,7 @@ bool LCHMFileImpl::getInfoFromSystem()
 	
 	int index = 0;
 	unsigned char* cursor = NULL, *p;
-	u_int16_t value = 0;
+	unsigned short value = 0;
 	long size = 0;
 
 	// Run the first loop to detect the encoding. We need this, because title could be
@@ -763,7 +796,7 @@ bool LCHMFileImpl::getInfoFromSystem()
 	buffer[size - 1] = 0;
 
 	// First loop to detect the encoding
-	for ( index = 0; index < (size - 1 - (long)sizeof(u_int16_t)) ;)
+	for ( index = 0; index < (size - 1 - (long)sizeof(unsigned short)) ;)
 	{
 		cursor = buffer + index;
 		value = UINT16ARRAY(cursor);
@@ -775,7 +808,7 @@ bool LCHMFileImpl::getInfoFromSystem()
 				cursor = buffer + index;
 			
 				if(m_topicsFile.isEmpty())
-					m_topicsFile = QString("/") + QString((const char*) buffer + index + 2);
+					m_topicsFile = QByteArray("/") + QByteArray((const char*) buffer + index + 2);
 				
 				break;
 			
@@ -784,7 +817,7 @@ bool LCHMFileImpl::getInfoFromSystem()
 				cursor = buffer + index;
 
 				if(m_indexFile.isEmpty())
-					m_indexFile = QString("/") + QString ((const char*)buffer + index + 2);
+					m_indexFile = QByteArray("/") + QByteArray((const char*)buffer + index + 2);
 				break;
 		
 			case 2:
@@ -792,13 +825,13 @@ bool LCHMFileImpl::getInfoFromSystem()
 				cursor = buffer + index;
 				
 				if(m_home.isEmpty() || m_home == "/")
-					m_home = QString("/") + QString ((const char*) buffer + index + 2);
+					m_home = QByteArray("/") + QByteArray((const char*) buffer + index + 2);
 				break;
 			
 			case 3:
 				index += 2;
 				cursor = buffer + index;
-				m_title = QString((const char*) (buffer + index + 2));
+				m_title = QByteArray( (const char*) (buffer + index + 2) );
 				break;
 
 			case 4:
@@ -814,19 +847,20 @@ bool LCHMFileImpl::getInfoFromSystem()
 				index += 2;
 				cursor = buffer + index;
 
-				if(m_topicsFile.isEmpty()) {
+				if ( m_topicsFile.isEmpty() )
+				{
 					QString topicAttempt = "/", tmp;
 					topicAttempt += QString ((const char*) buffer +index +2);
 
 					tmp = topicAttempt + ".hhc";
 				
 					if ( ResolveObject( tmp, &ui) )
-						m_topicsFile = tmp;
+						m_topicsFile = qPrintable( tmp );
 
 					tmp = topicAttempt + ".hhk";
 				
 					if ( ResolveObject( tmp, &ui) )
-						m_indexFile = tmp;
+						m_indexFile = qPrintable( tmp );
 				}
 				break;
 
@@ -850,26 +884,26 @@ bool LCHMFileImpl::getInfoFromSystem()
 }
 
  
-QCString LCHMFileImpl::convertSearchWord( const QString & src )
+QByteArray LCHMFileImpl::convertSearchWord( const QString & src )
 {
 	static const char * searchwordtable[128] =
 	{
 		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "s", 0, "oe", 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "s", 0, "oe", 0, 0, "y",
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
-  "a", "a", "a", "a", "a", "a", "ae", "c", "e", "e", "e", "e", "i", "i", "i", "i",
-  "d", "n", "o", "o", "o", "o", "o", 0, "o", "u", "u", "u", "u", "y", "\xDE", "ss",
-  "a", "a", "a", "a", "a", "a", "ae", "c", "e", "e", "e", "e", "i", "i", "i", "i",
-  "o", "n", "o", "o", "o", "o", "o", 0, "o", "u", "u", "u", "u", "y", "\xFE", "y"
+		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, "s", 0, "oe", 0, 0, "y",
+  		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+  		"a", "a", "a", "a", "a", "a", "ae", "c", "e", "e", "e", "e", "i", "i", "i", "i",
+  		"d", "n", "o", "o", "o", "o", "o", 0, "o", "u", "u", "u", "u", "y", "\xDE", "ss",
+  		"a", "a", "a", "a", "a", "a", "ae", "c", "e", "e", "e", "e", "i", "i", "i", "i",
+  		"o", "n", "o", "o", "o", "o", "o", 0, "o", "u", "u", "u", "u", "y", "\xFE", "y"
 	};
 
 	if ( !m_textCodec )
-		return (QCString) src.lower();
+		return (QByteArray) qPrintable( src.toLower() );
 
-	QCString dest = m_textCodec->fromUnicode (src);
+	QByteArray dest = m_textCodec->fromUnicode (src);
 
-	for ( unsigned int i = 0; i < dest.size(); i++ )
+	for ( int i = 0; i < dest.size(); i++ )
 	{
 		if ( dest[i] & 0x80 )
 		{
@@ -881,7 +915,7 @@ QCString LCHMFileImpl::convertSearchWord( const QString & src )
 		}
 	}
 
-	return dest.lower();
+	return dest.toLower();
 }
 
 
@@ -891,9 +925,9 @@ void LCHMFileImpl::getSearchResults( const LCHMSearchProgressResults& tempres,
 		  							 unsigned int limit_results )
 {
 	unsigned char combuf [COMMON_BUF_LEN];
-	QMap<u_int32_t, u_int32_t> urlsmap;  // used to prevent duplicated urls
+	QMap<unsigned int, unsigned int> urlsmap;  // used to prevent duplicated urls
 	
-	for ( unsigned int i = 0; i < tempres.size(); i++ )
+	for ( int i = 0; i < tempres.size(); i++ )
 	{
 		if ( urlsmap.find (tempres[i].urloff) != urlsmap.end() )
 			continue;
@@ -914,7 +948,7 @@ void LCHMFileImpl::getSearchResults( const LCHMSearchProgressResults& tempres,
 
 QString LCHMFileImpl::normalizeUrl( const QString & path ) const
 {
-	int pos = path.find ('#');
+	int pos = path.indexOf ('#');
 	QString fixedpath = pos == -1 ? path : path.left (pos);
 	
 	return LCHMUrlFactory::makeURLabsoluteIfNeeded( fixedpath );
@@ -925,7 +959,7 @@ QString LCHMFileImpl::normalizeUrl( const QString & path ) const
  * FIXME: <OBJECT type="text/sitemap"><param name="Merge" value="hhaxref.chm::/HHOCX_c.hhc"></OBJECT>
  *  (from htmlhelp.chm)
 */
-bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHMParsedEntry > * data, bool asIndex )
+bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QVector< LCHMParsedEntry > * data, bool asIndex )
 {
 	QString src;
 	const int MAX_NEST_DEPTH = 256;
@@ -939,7 +973,7 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 	// Save the index for debugging purposes
 	QFile outfile( "parsed.htm" );
 	
-	if ( outfile.open( IO_WriteOnly ) )
+	if ( outfile.open( QIODevice::WriteOnly ) )
 	{
 		QTextStream textstream( &outfile );
 		textstream << src;
@@ -957,7 +991,7 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 	// Split the HHC file by HTML tags
 	int stringlen = src.length();
 	
-	while ( pos < stringlen && (pos = src.find ('<', pos)) != -1 )
+	while ( pos < stringlen && (pos = src.indexOf ('<', pos)) != -1 )
 	{
 		int i, word_end = 0;
 		
@@ -967,10 +1001,10 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 			if ( (src[i] == '"' || src[i] == '\'') )
 			{
 				// find where quote ends, either by another quote, or by '>' symbol (some people don't know HTML)
-				int nextpos = src.find (src[i], i+1);
-				if ( nextpos == -1 	&& (nextpos = src.find ('>', i+1)) == -1 )
+				int nextpos = src.indexOf (src[i], i+1);
+				if ( nextpos == -1 	&& (nextpos = src.indexOf ('>', i+1)) == -1 )
 				{
-					qWarning ("LCHMFileImpl::ParseHhcAndFillTree: corrupted TOC: %s", src.mid(i).ascii());
+					qWarning ("LCHMFileImpl::ParseHhcAndFillTree: corrupted TOC: %s", qPrintable( src.mid(i) ));
 					return false;
 				}
 
@@ -985,19 +1019,19 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 		QString tagword, tag = src.mid (pos, i - pos);
 		 
 		if ( word_end )
-			tagword = src.mid (pos, word_end - pos).lower();
+			tagword = src.mid (pos, word_end - pos).toLower();
 		else
-			tagword = tag.lower();
+			tagword = tag.toLower();
 
-		//qDebug ("tag: '%s', tagword: '%s'\n", tag.ascii(), tagword.ascii());
+		//qDebug ("tag: '%s', tagword: '%s'\n", qPrintable( tag ), qPrintable( tagword ) );
 						
 		// <OBJECT type="text/sitemap"> - a topic entry
-		if ( tagword == "object" && tag.find ("text/sitemap", 0, false) != -1 )
+		if ( tagword == "object" && tag.indexOf ("text/sitemap", 0, Qt::CaseInsensitive ) != -1 )
 			in_object = true;
 		else if ( tagword == "/object" && in_object ) 
 		{
 			// a topic entry closed. Add a tree item
-			if ( entry.name )
+			if ( !entry.name.isEmpty() )
 			{
 				if ( !root_indent_offset_set )
 				{
@@ -1008,6 +1042,9 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 						qWarning("CHM has improper index; root indent offset is %d", root_indent_offset);
 				}
 
+				// Trim the entry name
+				entry.name = entry.name.trimmed();
+				
 				int real_indent = indent - root_indent_offset;
 				
 				entry.indent = real_indent;
@@ -1016,9 +1053,9 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 			else
 			{
 				if ( !entry.urls.isEmpty() )
-					qDebug ("LCHMFileImpl::ParseAndFillTopicsTree: <object> tag with url \"%s\" is parsed, but name is empty.", entry.urls[0].ascii());
+					qWarning ("LCHMFileImpl::ParseAndFillTopicsTree: <object> tag with url \"%s\" is parsed, but name is empty.", qPrintable( entry.urls[0] ));
 				else
-					qDebug ("LCHMFileImpl::ParseAndFillTopicsTree: <object> tag is parsed, but both name and url are empty.");	
+					qWarning ("LCHMFileImpl::ParseAndFillTopicsTree: <object> tag is parsed, but both name and url are empty.");	
 			}
 
 			entry.name = QString::null;
@@ -1033,20 +1070,20 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 			QString name_pattern = "name=", value_pattern = "value=";
 			QString pname, pvalue;
 
-			if ( (offset = tag.find (name_pattern, 0, FALSE)) == -1 )
-				qFatal ("LCHMFileImpl::ParseAndFillTopicsTree: bad <param> tag '%s': no name=\n", tag.ascii());
+			if ( (offset = tag.indexOf (name_pattern, 0, Qt::CaseInsensitive )) == -1 )
+				qFatal ("LCHMFileImpl::ParseAndFillTopicsTree: bad <param> tag '%s': no name=\n", qPrintable( tag ));
 
 			// offset+5 skips 'name='
 			offset = findStringInQuotes (tag, offset + name_pattern.length(), pname, TRUE, FALSE);
-			pname = pname.lower();
+			pname = pname.toLower();
 
-			if ( (offset = tag.find (value_pattern, offset, FALSE)) == -1 )
-				qFatal ("LCHMFileImpl::ParseAndFillTopicsTree: bad <param> tag '%s': no value=\n", tag.ascii());
+			if ( (offset = tag.indexOf(value_pattern, offset, Qt::CaseInsensitive )) == -1 )
+				qFatal ("LCHMFileImpl::ParseAndFillTopicsTree: bad <param> tag '%s': no value=\n", qPrintable( tag ));
 
 			// offset+6 skips 'value='
 			findStringInQuotes (tag, offset + value_pattern.length(), pvalue, FALSE, TRUE);
 
-			//qDebug ("<param>: name '%s', value '%s'", pname.ascii(), pvalue.ascii());
+			//qDebug ("<param>: name '%s', value '%s'", qPrintable( pname ), qPrintable( pvalue ));
 
 			if ( pname == "name" )
 			{
@@ -1054,12 +1091,31 @@ bool LCHMFileImpl::parseFileAndFillArray( const QString & file, QT34VECTOR< LCHM
 				if ( !pvalue.isEmpty() )
 					entry.name = pvalue;
 			}
+			else if ( pname == "merge" )
+			{
+				// MERGE implementation is experimental
+				QString mergeurl = LCHMUrlFactory::makeURLabsoluteIfNeeded( pvalue );
+				QString mergecontent;
+
+				if ( getFileContentAsString( &mergecontent, mergeurl ) && !mergecontent.isEmpty() )
+				{
+					qWarning( "MERGE is used in index; the implementation is experimental. Please let me know if it works" );
+
+					// Merge the read value into the current parsed file.
+					// To save memory it is done in a kinda hacky way:
+					src = mergecontent + src.mid( i );
+					pos = 0;
+					stringlen = src.length();
+				}
+				else
+					qWarning( "MERGE is used in index but file %s was not found in CHM archive", qPrintable(mergeurl) );
+			}
 			else if ( pname == "local" )
 			{
 				// Check for URL duplication
 				QString url = LCHMUrlFactory::makeURLabsoluteIfNeeded( pvalue );
 				
-				if ( entry.urls.find( url ) == entry.urls.end() )
+				if ( !entry.urls.contains( url ) )
 					entry.urls.push_back( url );
 			}
 			else if ( pname == "see also" && asIndex && entry.name != pvalue )
@@ -1104,14 +1160,20 @@ bool LCHMFileImpl::getFileContentAsBinary( QByteArray * data, const QString & ur
 	if( !ResolveObject( url, &ui ) )
 		return false;
 
-	data->resize( ui.length );
+	return getFileContentAsBinary( data, &ui );
+}
+
+
+bool LCHMFileImpl::getFileContentAsBinary(QByteArray * data, const chmUnitInfo * ui) const
+{
+	data->resize( ui->length );
 			
-	if ( RetrieveObject( &ui, (unsigned char*) data->data(), 0, ui.length ) )
+	if ( RetrieveObject( ui, (unsigned char*) data->data(), 0, ui->length ) )
 		return true;
 	else
 		return false;
 }
-
+	
 	
 bool LCHMFileImpl::getFileContentAsString( QString * str, const QString & url, bool internal_encoding )
 {
@@ -1126,7 +1188,7 @@ bool LCHMFileImpl::getFileContentAsString( QString * str, const QString & url, b
 			buf.resize( length + 1 );
 			buf [length] = '\0';
 			
-			*str = internal_encoding ? (QString)((const char*) buf) :  encodeWithCurrentCodec((const char*) buf);
+			*str = internal_encoding ? (QString)( buf.constData() ) :  encodeWithCurrentCodec( buf.constData() );
 			return true;
 		}
 	}
@@ -1142,7 +1204,7 @@ QString LCHMFileImpl::getTopicByUrl( const QString & url ) const
 	if ( it == m_url2topics.end() )
 		return QString::null;
 	
-	return it.data();
+	return it.value();
 }
 
 
@@ -1235,7 +1297,7 @@ void LCHMFileImpl::fillTopicsUrlMap()
 		return;
 
 	// Read those tables
-	QByteArray topics( m_chmTOPICS.length ), urltbl( m_chmURLTBL.length ), urlstr( m_chmURLSTR.length ), strings( m_chmSTRINGS.length );
+	QVector<unsigned char> topics( m_chmTOPICS.length ), urltbl( m_chmURLTBL.length ), urlstr( m_chmURLSTR.length ), strings( m_chmSTRINGS.length );
 
 	if ( !RetrieveObject( &m_chmTOPICS, (unsigned char*) topics.data(), 0, m_chmTOPICS.length )
 	|| !RetrieveObject( &m_chmURLTBL, (unsigned char*) urltbl.data(), 0, m_chmURLTBL.length )
@@ -1245,13 +1307,13 @@ void LCHMFileImpl::fillTopicsUrlMap()
 	
 	for ( unsigned int i = 0; i < m_chmTOPICS.length; i += TOPICS_ENTRY_LEN )
 	{
-		u_int32_t off_title = get_int32_le( (u_int32_t *)(topics.data() + i + 4) );
-		u_int32_t off_url = get_int32_le( (u_int32_t *)(topics.data() + i + 8) );
-		off_url = get_int32_le( (u_int32_t *)( urltbl.data() + off_url + 8) ) + 8;
+		unsigned int off_title = get_int32_le( (unsigned int *)(topics.data() + i + 4) );
+		unsigned int off_url = get_int32_le( (unsigned int *)(topics.data() + i + 8) );
+		off_url = get_int32_le( (unsigned int *)( urltbl.data() + off_url + 8) ) + 8;
 
 		QString url = LCHMUrlFactory::makeURLabsoluteIfNeeded( (const char*) urlstr.data() + off_url );
 
-		if ( off_title < strings.size() )
+		if ( off_title < (unsigned int)strings.size() )
 			m_url2topics[url] = encodeWithCurrentCodec ( (const char*) strings.data() + off_title );
 		else
 			m_url2topics[url] = "Untitled";
@@ -1269,3 +1331,339 @@ bool LCHMFileImpl::getFileSize(unsigned int * size, const QString & url)
 	*size = ui.length;
 	return true;
 }
+
+
+bool LCHMFileImpl::parseBinaryTOC( QVector< LCHMParsedEntry > * toc ) const
+{
+	if ( hasOption("nobintables") )
+		return false;
+	
+	if ( !m_lookupTablesValid )
+		return false;
+
+	QByteArray tocidx, topics, urltbl, urlstr, strings;
+	
+	// Read the index tables
+	if ( !getFileContentAsBinary( &tocidx, "/#TOCIDX" )
+	|| !getFileContentAsBinary( &topics, "/#TOPICS" )
+	|| !getFileContentAsBinary( &urltbl, "/#URLTBL" )
+	|| !getFileContentAsBinary( &urlstr, "/#URLSTR" )
+	|| !getFileContentAsBinary( &strings, "/#STRINGS" ) )
+		return false;
+
+	// Shamelessly stolen from xchm
+	if ( !RecurseLoadBTOC( tocidx, topics, urltbl, urlstr, strings, UINT32ARRAY( tocidx.data() ),  toc, 0 ) )
+	{
+		qWarning("Failed to parse binary TOC, fallback to text-based TOC");
+		toc->clear();
+		return false;
+	}
+
+	return true;
+}
+
+
+//
+// This piece of code was based on the one in xchm written by  Razvan Cojocaru <razvanco@gmx.net>
+//
+bool LCHMFileImpl::RecurseLoadBTOC( const QByteArray& tocidx,
+									const QByteArray& topics,
+									const QByteArray& urltbl,
+									const QByteArray& urlstr,
+									const QByteArray& strings,
+									int offset,
+		 							QVector< LCHMParsedEntry > * entries,
+		  							int level ) const
+{
+	while ( offset )
+	{
+		// If this is end of TOCIDX, return.
+		if ( tocidx.size() < offset + 20 )
+			return true;
+
+		unsigned int flags = UINT32ARRAY( tocidx.data() + offset + 4 );
+		int index = UINT32ARRAY( tocidx.data() + offset + 8 );
+	
+		if ( (flags & 0x04) || (flags & 0x08))
+		{
+			QString name, value;
+
+			if ( (flags & 0x08) == 0 )
+			{
+				if ( strings.size() < index + 1 )
+				{
+					qWarning("LCHMFile::RecurseLoadBTOC: invalid name index (%d) for book TOC entry!", index );
+					return false;
+				}
+
+				name = encodeWithCurrentCodec( strings.data() + index);
+			}
+			else
+			{
+				if ( topics.size() < (index * 16) + 12 )
+				{
+					qWarning("LCHMFile::RecurseLoadBTOC: invalid name index (%d) for local TOC entry!", index );
+					return false;
+				}
+
+				unsigned int tocoffset = UINT32ARRAY(topics.data()+ (index * 16) + 4);
+				long test = (long)tocoffset;
+
+				if ( (unsigned) strings.size() < tocoffset + 1 )
+				{
+					qWarning("LCHMFile::RecurseLoadBTOC: invalid name tocoffset (%d) for TOC entry!", tocoffset );
+					return false;
+				}
+
+				if ( test == -1 )
+				{
+					qWarning("LCHMFile::RecurseLoadBTOC: invalid name offset (%d) for TOC entry!", tocoffset );
+					return false;
+				}
+
+				name = encodeWithCurrentCodec( strings.data() + tocoffset );
+
+				// #URLTBL index
+				tocoffset = UINT32ARRAY( topics.data() + (index * 16) + 8 );
+			
+				if ( (unsigned) urltbl.size() < tocoffset + 12 )
+				{
+					qWarning("LCHMFile::RecurseLoadBTOC: invalid url index (%d) for TOC entry!", tocoffset );
+					return false;
+				}
+
+				tocoffset = UINT32ARRAY(urltbl.data() + tocoffset + 8);
+				
+				if ( (unsigned) urlstr.size() < tocoffset )
+				{
+					qWarning("LCHMFile::RecurseLoadBTOC: invalid url offset (%d) for TOC entry!", tocoffset );
+					return false;
+				}
+
+				value = encodeWithCurrentCodec( urlstr.data() + tocoffset + 8 );
+			}
+
+			LCHMParsedEntry entry;
+			entry.name = name.trimmed();
+
+			if ( !entry.name.isEmpty() )
+			{
+				if ( !value.isEmpty() )
+					entry.urls.push_back( LCHMUrlFactory::makeURLabsoluteIfNeeded( value ) );
+
+				entry.imageid = LCHMBookIcons::IMAGE_AUTO;
+				entry.indent = level;
+				entries->push_back( entry );
+			}
+		}
+
+		if ( flags & 0x04 )
+		{
+			// book
+			if ( tocidx.size() < offset + 24 )
+			{
+				qWarning("LCHMFile::RecurseLoadBTOC: invalid child entry offset (%d)", offset );
+				return false;
+			}
+
+			unsigned int childoffset = UINT32ARRAY( tocidx.data() + offset + 20 );
+			
+			if ( childoffset )
+			{
+				if ( !RecurseLoadBTOC( tocidx, topics, urltbl, urlstr, strings, childoffset, entries, level + 1 ) )
+					return false;
+			}
+		}
+		
+		offset = UINT32ARRAY( tocidx.data() + offset + 0x10 );
+	}
+
+	return true;
+}
+
+
+
+bool LCHMFileImpl::hasOption(const QString & name) const
+{
+	if ( !m_envOptions.isEmpty() && m_envOptions.contains( name ) )
+		return true;
+
+	return false;
+}
+
+
+//
+// This piece of code was based on the one in xchm written by Razvan Cojocaru <razvanco@gmx.net>
+//
+bool LCHMFileImpl::parseBinaryIndex( QVector< LCHMParsedEntry > * entries ) const
+{
+	if ( !m_lookupTablesValid )
+		return false;
+
+	if ( hasOption("nobintables") )
+		return false;
+	
+	if ( !loadBinaryIndex( entries ) )
+	{
+		qWarning("Failed to parse binary index, fallback to text-based index");
+		entries->clear();
+		return false;
+	}
+
+	return true;
+}
+
+
+QString LCHMFileImpl::getBtreeString( const QByteArray& btidx, unsigned long * offset, unsigned short * spaceLeft ) const
+{
+	QByteArray string;
+	unsigned short tmp;
+	
+	while ( 1 )
+	{
+		// accumulate the name
+		if ( (unsigned) btidx.size() < *offset + sizeof(unsigned short) )
+			return QString();
+				
+		tmp = UINT16ARRAY( btidx.data() + *offset );
+		*offset += sizeof(unsigned short);
+		*spaceLeft -= sizeof(unsigned short);
+
+		if ( tmp == 0x00 )
+			break;
+
+		string.push_back( tmp );
+	}
+
+	return encodeWithCurrentCodec( string ).trimmed();
+}
+
+
+bool LCHMFileImpl::loadBinaryIndex( QVector< LCHMParsedEntry > * entries ) const
+{
+	QByteArray btidx, topics, urltbl, urlstr, strings;
+	
+	// Read the index tables
+	if ( !getFileContentAsBinary( &btidx, "/$WWKeywordLinks/BTree" )
+	|| !getFileContentAsBinary( &topics, "/#TOPICS" )
+	|| !getFileContentAsBinary( &urltbl, "/#URLTBL" )
+	|| !getFileContentAsBinary( &urlstr, "/#URLSTR" )
+	|| !getFileContentAsBinary( &strings, "/#STRINGS" ) )
+		return false;
+
+	// Make sure we have enough entries in tree
+	if ( btidx.size() < 88 )
+	{
+		qWarning("LCHMFile::loadBinaryIndex: BTree is too small" );
+		return false;
+	}
+
+	unsigned long offset = 0x4c;
+	int next = -1;
+	unsigned short freeSpace, spaceLeft;
+	const short blockSize = 2048;
+	bool found_item = false;
+
+	do
+	{
+		if ( (unsigned) btidx.size() < offset + 12 )
+			break;
+
+		freeSpace = UINT16ARRAY( btidx.data() + offset );
+		next = INT32ARRAY( btidx.data() + offset + 8 );
+		spaceLeft = blockSize - 12;
+		offset += 12;
+
+		while ( spaceLeft > freeSpace )
+		{
+			QString value;
+			LCHMParsedEntry entry;
+
+			entry.name = getBtreeString( btidx, &offset, &spaceLeft );
+
+			if ( entry.name.isEmpty() )
+			{
+				qWarning("LCHMFile::loadBinaryIndex: cannot parse name" );
+				return false;
+			}
+			
+			if ( (unsigned) btidx.size() < offset + 16 )
+			{
+				qWarning("LCHMFile::loadBinaryIndex: index is terminated by name" );
+				return false;
+			}
+
+			unsigned short seeAlso = UINT16ARRAY(btidx.data() + offset);
+			unsigned int numTopics = UINT32ARRAY(btidx.data() + offset + 0xc);
+			offset += 16;
+			spaceLeft -= 16;
+
+			if ( seeAlso )
+			{
+				QString seealso = getBtreeString( btidx, &offset, &spaceLeft );
+				
+				if ( entry.name != seealso )
+					entry.urls.push_back( ":" + seealso );
+			}
+			else
+			{
+				for ( unsigned int i = 0; i < numTopics && spaceLeft > freeSpace; ++i )
+				{
+					if ( (unsigned) btidx.size() < offset + sizeof(unsigned int) )
+					{
+						qWarning("LCHMFile::loadBinaryIndex: premature url termination" );
+						return false;
+					}
+
+					unsigned int index = UINT32ARRAY( btidx.data() + offset );
+
+					// #URLTBL index
+					unsigned int tocoffset = UINT32ARRAY( topics.data() + (index * 16) + 8 );
+			
+					if ( (unsigned) urltbl.size() < tocoffset + 12 )
+					{
+						qWarning("LCHMFile::loadBinaryIndex: invalid url index (%d) for TOC entry!", tocoffset );
+						return false;
+					}
+
+					tocoffset = UINT32ARRAY(urltbl.data() + tocoffset + 8);
+				
+					if ( (unsigned) urlstr.size() < tocoffset )
+					{
+						qWarning("LCHMFile::loadBinaryIndex: invalid url offset (%d) for TOC entry!", tocoffset );
+						return false;
+					}
+
+					QString url = encodeWithCurrentCodec( urlstr.data() + tocoffset + 8 );
+					entry.urls.push_back( LCHMUrlFactory::makeURLabsoluteIfNeeded( url ) );
+					offset += sizeof(unsigned int);
+					spaceLeft -= sizeof(unsigned int);
+				}
+			}
+
+			entry.name = entry.name.trimmed();
+
+			if ( !entry.name.isEmpty() )
+			{
+				entry.imageid = LCHMBookIcons::IMAGE_INDEX;
+				entry.indent = 0;
+				found_item = true;
+				entries->push_back( entry );
+			}
+					
+			if ( (unsigned) btidx.size() < offset + 8 )
+			{
+				qWarning("LCHMFile::loadBinaryIndex: binary index is gone" );
+				return false;
+			}
+
+			offset += 8;
+			spaceLeft -= 8;
+		}
+		
+		offset += spaceLeft;
+
+	} while ( next != -1 );
+
+	return found_item;
+}
