@@ -5,7 +5,7 @@
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -16,9 +16,11 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
+#include <QTextStream>
+
 #include "kde-qt.h"
 #include "kchmconfig.h"
 #include "kchmsettings.h"
@@ -33,29 +35,35 @@ const char * APP_PATHINUSERDIR = ".kchmviewer";
 KCHMConfig::KCHMConfig()
 {
 	QDir dir;
-	m_datapath = QDir::homeDirPath() + "/" + APP_PATHINUSERDIR;
+	m_datapath = QDir::homePath () + "/" + APP_PATHINUSERDIR;
 	 
 	dir.setPath (m_datapath);
 	
 	if ( !dir.exists() && !dir.mkdir(m_datapath) )
-		qWarning ("Could not create directory %s", m_datapath.ascii());
+		qWarning( "Could not create directory %s", qPrintable( m_datapath ));
 
 	m_LoadLatestFileOnStartup = false;
 	m_onNewChmClick = ACTION_ASK_USER;
 	m_onExternalLinkClick = ACTION_ASK_USER;
-	m_HistorySize = 10;
+	m_numOfRecentFiles = 10;
 	m_HistoryStoreExtra = true;
-	m_useSearchEngine = SEARCH_USE_MINE;
 	
-	m_QtBrowserPath = "viewurl-netscape.sh '%s'";
-	m_kdeUseQTextBrowser = false;
+	// Webkit only present in 4.4+
+#if defined (QT_WEBKIT_LIB)
+	m_usedBrowser = BROWSER_QTWEBKIT;
+#else
+	m_usedBrowser = BROWSER_QTEXTBROWSER;
+#endif
+		
 	m_kdeEnableJS = false;
 	m_kdeEnableJava = false;
 	m_kdeEnablePlugins = true;
 	m_kdeEnableRefresh = false;
 	
 	m_advUseInternalEditor = true;
-	m_advExternalEditorPath = "kate '%s'";
+	m_advLayoutDirectionRL = false;
+	m_advAutodetectEncoding = false;
+	m_advExternalEditorPath = "/usr/bin/kate";
 	
 	m_lastOpenedDir = "";
 }
@@ -68,17 +76,17 @@ KCHMConfig::~KCHMConfig()
 bool KCHMConfig::load()
 {
 	QFile file (m_datapath + "/config");
-	if ( !file.open (IO_ReadOnly) )
+	if ( !file.open (QIODevice::ReadOnly) )
 		return false; // no error message - not actually a problem
 	
 	QString line;
 	char readbuf[4096];
 	bool getting_history = false;
-	m_History.clear();
+	m_recentFiles.clear();
 	
 	while ( file.readLine( readbuf, sizeof(readbuf) - 1 ) > 0 )
 	{
-		line = QString::fromUtf8( readbuf ).stripWhiteSpace();
+		line = QString::fromUtf8( readbuf ).trimmed();
 		
 		// skip empty lines and comments
 		if ( line.isEmpty() || line[0] == '#' )
@@ -86,18 +94,18 @@ bool KCHMConfig::load()
 		
 		QRegExp rxsection ("^\\[(\\w+)\\]$"), rxkeypair ("^(\\w+)\\s*=\\s*(.*)$");
 		
-		if ( rxsection.search ( line ) != -1 )
+		if ( rxsection.indexIn( line ) != -1 )
 		{
 			if ( rxsection.cap (1) == "settings" )
 				getting_history = false;
 			else if ( rxsection.cap (1) == "history" )
 				getting_history = true;
 			else
-				qWarning ("Unknown configuration section: %s", rxsection.cap (1).ascii());
+				qWarning ("Unknown configuration section: %s", qPrintable( rxsection.cap(1) ));
 			
 			continue;
 		}
-		else if ( !getting_history && rxkeypair.search ( line ) != -1 )
+		else if ( !getting_history && rxkeypair.indexIn( line ) != -1 )
 		{
 			QString key (rxkeypair.cap (1)), value (rxkeypair.cap(2));
 			
@@ -108,13 +116,11 @@ bool KCHMConfig::load()
 			else if ( key == "onExternalLinkClick" )
 				m_onExternalLinkClick = (choose_action_t) value.toInt();
 			else if ( key == "HistorySize" )
-				m_HistorySize = value.toInt();
+				m_numOfRecentFiles = value.toInt();
 			else if ( key == "HistoryStoreExtra" )
 				m_HistoryStoreExtra = value.toInt() ? true : false;
-			else if ( key == "QtBrowserPath" )
-				m_QtBrowserPath = value;
-			else if ( key == "kdeUseQTextBrowser" )
-				m_kdeUseQTextBrowser = value.toInt() ? true : false;
+			else if ( key == "UsedBrowser" )
+				m_usedBrowser = value.toInt();
 			else if ( key == "kdeEnableJS" )
 				m_kdeEnableJS = value.toInt() ? true : false;
 			else if ( key == "kdeEnableJava" )
@@ -129,87 +135,85 @@ bool KCHMConfig::load()
 				m_advUseInternalEditor = value.toInt() ? true : false;
 			else if ( key == "advExternalEditorPath" )
 				m_advExternalEditorPath = value;
-			else if ( key == "useSearchEngine" )
-				m_useSearchEngine = (use_search_engine) value.toInt();
+			else if ( key == "advLayoutDirectionRL" )
+				m_advLayoutDirectionRL = value.toInt() ? true : false;
+			else if ( key == "advAutoDetectEncoding" )
+				m_advAutodetectEncoding = value.toInt() ? true : false;
+			else if ( key == "useSearchEngine" || key == "QtBrowserPath" )
+				// Do nothing; not used anymore
+				;
 			else
-				qWarning ("Unknown key=value pair: %s", line.ascii());
+				qWarning ("Unknown key=value pair: %s", qPrintable( line ));
 		}
 		else if ( getting_history )
 		{
-			if ( m_History.size() < m_HistorySize )
-				addFileToHistory( line );
+			if ( m_recentFiles.size() < m_numOfRecentFiles )
+				addRecentFile( line );
 		}
 		else
-			qWarning ("Unknown line in configuration: %s", line.ascii());
+			qWarning ("Unknown line in configuration: %s", qPrintable( line ));
 	}
 
+	// Reset webkit browser to qtextbrowser when older version is running
+#if !defined (QT_WEBKIT_LIB)
+	if ( m_usedBrowser == BROWSER_QTWEBKIT )
+		m_usedBrowser = BROWSER_QTEXTBROWSER;
+#endif
+	
 	return true;
 }
 
 bool KCHMConfig::save( )
 {
-	QFile file (m_datapath + "/config");
-	if ( !file.open (IO_WriteOnly) )
+	QFile file( m_datapath + "/config" );
+	if ( !file.open (QIODevice::WriteOnly) )
 	{
-		qWarning ("Could not write settings into file %s: %s", file.name().ascii(), file.errorString().ascii());
+		qWarning( "Could not write settings into file %s: %s", 
+		          qPrintable( file.fileName() ), 
+		          qPrintable(  file.errorString() ) );
 		return false;
 	}
 	
 	QTextStream stream( &file );
-	stream.setEncoding( QTextStream::UnicodeUTF8 );
+	stream.setCodec( "UTF-8" );
 	stream << "[settings]\n";
 	stream << "LoadLatestFileOnStartup=" << m_LoadLatestFileOnStartup << "\n";
 
 	stream << "onNewChmClick=" << m_onNewChmClick << "\n";
 	stream << "onExternalLinkClick=" << m_onExternalLinkClick << "\n";
-	stream << "HistorySize=" << m_HistorySize << "\n";
+	stream << "HistorySize=" << m_numOfRecentFiles << "\n";
 	stream << "HistoryStoreExtra=" << m_HistoryStoreExtra << "\n";
-
-	stream << "QtBrowserPath=" << m_QtBrowserPath << "\n";
-	stream << "kdeUseQTextBrowser=" << m_kdeUseQTextBrowser << "\n";
+	stream << "UsedBrowser=" << m_usedBrowser << "\n";
+	
 	stream << "kdeEnableJS=" << m_kdeEnableJS << "\n";
 	stream << "kdeEnableJava=" << m_kdeEnableJava << "\n";
 	stream << "kdeEnablePlugins=" << m_kdeEnablePlugins << "\n";
 	stream << "kdeEnableRefresh=" << m_kdeEnableRefresh << "\n";
 	stream << "advUseInternalEditor=" << m_advUseInternalEditor << "\n";
 	stream << "advExternalEditorPath=" << m_advExternalEditorPath << "\n";
-	stream << "useSearchEngine=" << m_useSearchEngine << "\n";
+	stream << "advLayoutDirectionRL=" << m_advLayoutDirectionRL << "\n";
+	stream << "advAutoDetectEncoding=" << m_advAutodetectEncoding << "\n";
 	
 	stream << "LastOpenedDir=" << m_lastOpenedDir << "\n";	
 	
 	stream << "\n[history]\n";
 	
 	// Do not write all the history, but only the needed amount
-	for ( unsigned int i = 0; i < m_History.size(); i++ )
-		stream << m_History[m_History.size() - 1 - i] << "\n";
+	for ( int i = 0; i < m_recentFiles.size(); i++ )
+		stream << m_recentFiles[m_recentFiles.size() - 1 - i] << "\n";
 	
-	//	m_History
 	return true;
 }
 
-void KCHMConfig::addFileToHistory( const QString & file )
+void KCHMConfig::addRecentFile( const QString & filename )
 {
-	QStringList::Iterator itr = m_History.find( file );
+	m_recentFiles.removeAll( filename );
+	m_recentFiles.prepend( filename );
 	
-	// check whether file already exists in history - more it forward
-	if ( itr != m_History.end() )
+	while( m_recentFiles.size() > m_numOfRecentFiles )
 	{
-		m_History.erase( itr );
-		m_History.push_back(file);
-		return;
+		// Remove the appropriate history file
+		mainWindow->currentSettings()->removeSettings( m_recentFiles.last() );
+		m_recentFiles.removeLast();
 	}
-
-	if ( m_History.size() < m_HistorySize )
-	{
-		m_History.push_back( file );
-		return;
-	}
-	
-	// Remove a file from the front
-	QString filetoremove = m_History[0];
-	m_History.erase( m_History.begin() );
-	m_History.push_back( file );
-	
-	// And remove the appropriate history file
-	mainWindow->currentSettings()->removeSettings ( filetoremove );
 }
