@@ -5,7 +5,7 @@
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -16,7 +16,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
 #ifndef INCLUDE_KCHMCONTENTSWINDOW_H
@@ -24,29 +24,34 @@
 
 #include "kde-qt.h"
 #include "kchmtreeviewitem.h"
+#include "ui_tab_contents.h"
 
-/**
-@author tim
-*/
-class KCHMContentsWindow : public KQListView
+
+class KCHMContentsWindow : public QWidget, public Ui::TabContents
 {
 	Q_OBJECT
 	public:
-    	KCHMContentsWindow( QWidget *parent = 0, const char *name = 0 );
+    	KCHMContentsWindow( QWidget *parent = 0 );
 		~KCHMContentsWindow();
 		
-		QRect	tip( const QPoint & p );
-		
 		void	refillTableOfContents();
+		void	showItem( KCHMIndTocItem * item );
+		void	search( const QString& text );
 		
 		KCHMIndTocItem *	getTreeItem( const QString& url );
 		
 	public slots:
-		void	slotContextMenuRequested ( QListViewItem *item, const QPoint &point, int column );
-		
+		void	onContextMenuRequested ( const QPoint &point );
+		void	onClicked ( QTreeWidgetItem * item, int column );
+	
 	private:
-		KQPopupMenu 				  * m_contextMenu;
+		virtual void showEvent ( QShowEvent * );
+	
+	private:
+		bool		m_contentFilled;
+		QMenu 	*	m_contextMenu;
 		QMap<QString, KCHMIndTocItem*>	m_urlListMap;
 };
 
+
 #endif /* INCLUDE_KCHMCONTENTSWINDOW_H */
