@@ -5,7 +5,7 @@
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -16,18 +16,19 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
-#include <qfile.h>
-#include <qfileinfo.h>
-#include <qdatastream.h>
+#include <QFile>
+#include <QFileInfo>
+#include <QDataStream>
+#include <QDateTime>
  
 #include "kchmsettings.h"
 #include "kchmconfig.h"
 
-static Q_INT32 SETTINGS_MAGIC = 0xD8AB4E76;
-static Q_INT32 SETTINGS_VERSION = 4;
+static qint32 SETTINGS_MAGIC = 0xD8AB4E76;
+static qint32 SETTINGS_VERSION = 4;
 
 /*
  * The order is important!
@@ -50,6 +51,8 @@ enum marker_t
  	MARKER_CONTENTSDATA,
 	MARKER_INDEXDATA,
 
+	MARKER_ACTIVEENCODINGNAME,
+		
 	// This should be the last
 	MARKER_END = 0x7FFF
 };
@@ -84,7 +87,7 @@ static inline QDataStream& operator<< ( QDataStream& s, const KCHMSettings::Save
 
 static inline QDataStream& operator>> ( QDataStream& s, KCHMSettings::SavedViewWindow& b )
 {
-	Q_INT32 version;
+	qint32 version;
 	
 	s >> version; 
 	s >> b.url;
@@ -98,7 +101,7 @@ KCHMSettings::KCHMSettings( )
 {
 	m_activetabsystem = 0;
 	m_activetabwindow = 0;
-	m_activeencodinglcid = 0;
+	m_activeEncoding = "CP1252";
 	
 	m_window_size_x = 700;
 	m_window_size_y = 500;
@@ -110,7 +113,7 @@ bool KCHMSettings::loadSettings( const QString & filename )
 {
 	m_activetabsystem = 0;
 	m_activetabwindow = 0;
-	m_activeencodinglcid = 0;
+	m_activeEncoding = "CP1252";
 	
 	m_searchhistory.clear();
 	m_bookmarks.clear();
@@ -119,8 +122,7 @@ bool KCHMSettings::loadSettings( const QString & filename )
 	QFileInfo finfo ( filename );
 
 	m_settingsFile = QString::null;
-	m_searchDictFile = QString::null;
-	m_searchDocFile = QString::null;
+	m_searchIndex = QString::null;
 	
 	if ( !finfo.size() )
 		return false;
@@ -129,30 +131,30 @@ bool KCHMSettings::loadSettings( const QString & filename )
 	m_currentfilesize = finfo.size();
 	m_currentfiledate = finfo.lastModified().toTime_t();
 	
-	getFilenames( filename, &m_settingsFile, &m_searchDictFile, &m_searchDocFile );
+	getFilenames( filename, &m_settingsFile, &m_searchIndex );
 	
 	QFile file( m_settingsFile );
 
-    if ( !file.open (IO_ReadOnly) )
+    if ( !file.open (QIODevice::ReadOnly) )
 		return false; // it's ok, file may not exist
 	
     QDataStream stream (&file);
 
 	// Read and check header
-	Q_INT32 data;
+	qint32 data;
 	bool complete_read = false;
 	stream >> data; // magic
 	
 	if ( data != SETTINGS_MAGIC )
 	{
-		qWarning ("file %s has bad magic value, ignoring it.", file.name().ascii());
+		qWarning ("file %s has bad magic value, ignoring it.", qPrintable( file.fileName()) );
 		return false;
 	}
 	
 	stream >> data; // version
 	if ( data > SETTINGS_VERSION )
 	{
-		qWarning ("file %s has unsupported data version %d,  ignoring it.", file.name().ascii(), data);
+		qWarning ("file %s has unsupported data version %d, ignoring it.", qPrintable( file.fileName()), data);
 		return false;
 	}
 
@@ -194,8 +196,13 @@ bool KCHMSettings::loadSettings( const QString & filename )
 			stream >> m_activetabwindow;
 			break;
 			
+		// Not used anymore
 		case MARKER_ACTIVEENCODING:
-			stream >> m_activeencodinglcid;
+			stream >> data;
+			break;
+			
+		case MARKER_ACTIVEENCODINGNAME:
+			stream >> m_activeEncoding;
 			break;
 	
 		case MARKER_WINDOW_SIZE:
@@ -225,9 +232,11 @@ bool KCHMSettings::loadSettings( const QString & filename )
 bool KCHMSettings::saveSettings( )
 {
 	QFile file( m_settingsFile );
-    if ( !file.open (IO_WriteOnly) )
+    if ( !file.open (QIODevice::WriteOnly) )
 	{
-		qWarning ("Could not write settings into file %s: %s", file.name().ascii(), file.errorString().ascii());
+		qWarning ("Could not write settings into file %s: %s", 
+		          qPrintable( file.fileName()), 
+		          qPrintable( file.errorString() ));
 		return false;
 	}
 	
@@ -251,8 +260,8 @@ bool KCHMSettings::saveSettings( )
 	stream << MARKER_ACTIVETABWINDOW;
 	stream << m_activetabwindow;
 	
-	stream << MARKER_ACTIVEENCODING;
-	stream << m_activeencodinglcid;
+	stream << MARKER_ACTIVEENCODINGNAME;
+	stream << m_activeEncoding;
 	
 	// Save search history vector
 	stream << MARKER_SEARCHHISTORY;
@@ -277,22 +286,20 @@ bool KCHMSettings::saveSettings( )
 
 void KCHMSettings::removeSettings( const QString & filename )
 {
-	QString settingsfile, dictfile, doclistfile;
+	QString settingsfile, idxfile;
 	
-	getFilenames( filename, &settingsfile, &dictfile, &doclistfile );
+	getFilenames( filename, &settingsfile, &idxfile );
 	
 	QFile::remove( settingsfile );
-	QFile::remove( dictfile );
-	QFile::remove( doclistfile );
+	QFile::remove( idxfile );
 }
 
 
-void KCHMSettings::getFilenames(const QString & helpfilename, QString * settingsfile, QString * dictfile, QString * doclistfile )
+void KCHMSettings::getFilenames(const QString & helpfilename, QString * settingsfile, QString * indexfile )
 {
 	QFileInfo finfo ( helpfilename );
 	QString prefix = appConfig.m_datapath + "/" + finfo.baseName();
 
 	*settingsfile = prefix + ".kchmviewer";
-	*dictfile = prefix + ".dict";
-	*doclistfile  = prefix + ".doclist";
+	*indexfile = prefix + ".idx";
 }
