@@ -5,7 +5,7 @@
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -16,21 +16,14 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
 #ifndef KCHMSETTINGS_H
 #define KCHMSETTINGS_H
 
-#include <qstring.h>
-#include <qvaluelist.h>
-
-
-/**
-@author Georgy Yunaev
-*/
-class SavedBookmark;
-class QFileInfo;
+#include <QString>
+#include <QList>
 
 
 class KCHMSettings
@@ -42,14 +35,13 @@ class KCHMSettings
 		bool	saveSettings ( );
 		void 	removeSettings ( const QString& filename );
 		
-		QString	searchIndexDictFilename() const	{ return m_searchDictFile; }
-		QString	searchIndexDocFilename() const	{ return m_searchDocFile; }
+		QString	searchIndexFile() const	{ return m_searchIndex; }
 		
 		class SavedBookmark
 		{
 		public:
 			SavedBookmark() { scroll_y = 0; }
-			SavedBookmark (QString n, QString u, int y) : name(n), url(u), scroll_y(y) {};
+			SavedBookmark ( const QString& n, const QString& u, int y) : name(n), url(u), scroll_y(y) {};
 			
 			QString		name;
 			QString		url;
@@ -60,35 +52,34 @@ class KCHMSettings
 		{
 			public:
 				SavedViewWindow() { scroll_y = 0; zoom = 0; }
-				SavedViewWindow (QString u, int y, int z) : url(u), scroll_y(y), zoom(z) {};
+				SavedViewWindow ( const QString& u, int y, int z) : url(u), scroll_y(y), zoom(z) {};
 			
 				QString		url;
 				int			scroll_y;
 				int			zoom;
 		};
 		
-		typedef 	QValueList<QString>			search_saved_settings_t;
-		typedef 	QValueList<SavedBookmark>	bookmark_saved_settings_t;
-		typedef 	QValueList<SavedViewWindow>	viewindow_saved_settings_t;
+		typedef QList<QString>			search_saved_settings_t;
+		typedef QList<SavedBookmark>	bookmark_saved_settings_t;
+		typedef QList<SavedViewWindow>	viewindow_saved_settings_t;
 		
 		int							m_window_size_x;
 		int							m_window_size_y;
 		int							m_window_size_splitter;
 		int							m_activetabsystem;
 		int							m_activetabwindow;
-		int							m_activeencodinglcid;
+		QString						m_activeEncoding;
 		search_saved_settings_t		m_searchhistory;
 		bookmark_saved_settings_t	m_bookmarks;
 		viewindow_saved_settings_t	m_viewwindows;
 	
 	private:
-		void		getFilenames(const QString & helpfilename, QString * settingsfile, QString * dictfile, QString * doclistfile );
+		void		getFilenames(const QString & helpfilename, QString * settingsfile, QString * indexfile );
 		
 		unsigned int				m_currentfilesize;
 		unsigned int				m_currentfiledate;
 		QString						m_settingsFile;
-		QString						m_searchDictFile;
-		QString						m_searchDocFile;
+		QString						m_searchIndex;
 };
 
 #endif
