@@ -5,7 +5,7 @@
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
- *   the Free Software Foundation; either version 2 of the License, or     *
+ *   the Free Software Foundation; either version 3 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
  *   This program is distributed in the hope that it will be useful,       *
@@ -16,7 +16,7 @@
  *   You should have received a copy of the GNU General Public License     *
  *   along with this program; if not, write to the                         *
  *   Free Software Foundation, Inc.,                                       *
- *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
 
 // Do not use tr() or i18n() in this file - this class is not derived from QObject.
@@ -32,7 +32,6 @@
 #include "kchmviewwindow.h"
 #include "kchmmainwindow.h"
 #include "kchmviewwindowmgr.h"
-#include "kchmnavtoolbar.h"
 
 
 KCHMViewWindow::KCHMViewWindow( QTabWidget * parent )
@@ -71,26 +70,26 @@ QString KCHMViewWindow::makeURLabsolute ( const QString & url, bool set_as_base
 	&& !LCHMUrlFactory::isJavascriptURL (url)
 	&& !LCHMUrlFactory::isNewChmURL (url, p1, p2) )
 	{
-		newurl = QDir::cleanDirPath (url);
+		newurl = QDir::cleanPath (url);
 
 		// Normalize url, so it becomes absolute
 		if ( newurl[0] != '/' )
 			newurl = m_base_url + "/" + newurl;
 	
-		newurl = QDir::cleanDirPath (newurl);
+		newurl = QDir::cleanPath (newurl);
 
 		if ( set_as_base )
 		{
 			m_base_url = newurl;
 		
 			// and set up new baseurl
-			int i = newurl.findRev('/');
+			int i = newurl.lastIndexOf('/');
 			if ( i != -1 )
-				m_base_url = QDir::cleanDirPath (newurl.left (i + 1));
+				m_base_url = QDir::cleanPath (newurl.left (i + 1));
 		}
 	}
 
-//qDebug ("KCHMViewWindow::makeURLabsolute (%s) -> (%s)", url.ascii(), newurl.ascii());
+	//qDebug ("KCHMViewWindow::makeURLabsolute (%s) -> (%s)", url.ascii(), newurl.ascii());
 	return newurl;
 }
 
@@ -98,7 +97,7 @@ bool KCHMViewWindow::openUrl ( const QString& origurl )
 {
 	QString chmfile, page, newurl = origurl;
 
-	if ( !origurl )
+	if ( origurl.isEmpty() )
 		return true;
 
 	// URL could be a complete ms-its link. The file should be already loaded (for QTextBrowser),
@@ -106,9 +105,10 @@ bool KCHMViewWindow::openUrl ( const QString& origurl )
 	if ( LCHMUrlFactory::isNewChmURL( newurl, chmfile, page ) )
 	{
 		// If a new chm file is opened here, and we do not use KCHMLPart, we better abort
-		if ( chmfile != ::mainWindow->getOpenedFileName() && appConfig.m_kdeUseQTextBrowser )
+		if ( chmfile != ::mainWindow->getOpenedFileBaseName() && appConfig.m_usedBrowser != KCHMConfig::BROWSER_KHTMLPART )
 			qFatal("KCHMViewWindow::openUrl(): opened new chm file %s while current is %s",
-				   chmfile.ascii(), ::mainWindow->getOpenedFileName().ascii() );
+				   qPrintable( chmfile ),
+				   qPrintable( ::mainWindow->getOpenedFileName() ) );
 
 		// It is OK to have a new file in chm for KHTMLPart
 		newurl = page;
@@ -122,6 +122,11 @@ bool KCHMViewWindow::openUrl ( const QString& origurl )
 		m_newTabLinkKeeper = QString::null;
 		m_openedPage = newurl;
 		
+		// If m_openedPage contains #, strip it, and everything after it
+		int hash = m_openedPage.indexOf( '#' );
+		if ( hash != -1 )
+			m_openedPage = m_openedPage.left( hash );
+			
 		mainWindow->viewWindowMgr()->setTabName( this );
 		return true;
 	}
@@ -132,29 +137,29 @@ bool KCHMViewWindow::openUrl ( const QString& origurl )
 void KCHMViewWindow::handleStartPageAsImage( QString & link )
 {
 	// Handle pics
-	if ( link.endsWith( ".jpg", false )
-	|| link.endsWith( ".jpeg", false )
-	|| link.endsWith( ".gif", false )
-	|| link.endsWith( ".png", false )
-	|| link.endsWith( ".bmp", false ) )
+	if ( link.endsWith( ".jpg", Qt::CaseInsensitive )
+	|| link.endsWith( ".jpeg", Qt::CaseInsensitive )
+	|| link.endsWith( ".gif", Qt::CaseInsensitive )
+	|| link.endsWith( ".png", Qt::CaseInsensitive )
+	|| link.endsWith( ".bmp", Qt::CaseInsensitive ) )
 		link += LCHMUrlFactory::getInternalUriExtension();
 }
 
 
-KQPopupMenu * KCHMViewWindow::createStandardContextMenu( QWidget * parent )
+QMenu * KCHMViewWindow::createStandardContextMenu( QWidget * parent )
 {
-	KQPopupMenu * contextMenu = new KQPopupMenu( parent );
+	QMenu * contextMenu = new QMenu( parent );
 	
-	contextMenu->insertItem ( "&Copy", ::mainWindow, SLOT(slotBrowserCopy()) );
-	contextMenu->insertItem ( "&Select all", ::mainWindow, SLOT(slotBrowserSelectAll()) );
+	contextMenu->addAction( "&Copy", ::mainWindow, SLOT(slotBrowserCopy()) );
+	contextMenu->addAction( "&Select all", ::mainWindow, SLOT(slotBrowserSelectAll()) );
 		
 	return contextMenu;
 }
 
 
-KQPopupMenu * KCHMViewWindow::getContextMenu( const QString & link, QWidget * parent )
+QMenu * KCHMViewWindow::getContextMenu( const QString & link, QWidget * parent )
 {
-	if ( link.isNull() )
+	if ( link.isEmpty() )
 	{
 		// standard context menu
 		if ( !m_contextMenu )
@@ -169,11 +174,11 @@ KQPopupMenu * KCHMViewWindow::getContextMenu( const QString & link, QWidget * pa
 		if ( !m_contextMenuLink )
 		{
 			m_contextMenuLink = createStandardContextMenu( parent );
-			m_contextMenuLink->insertSeparator();
+			m_contextMenuLink->addSeparator();
 			
-			m_contextMenuLink->insertItem ( "&Open this link in a new tab", ::mainWindow, SLOT(slotOpenPageInNewTab()), QKeySequence("Shift+Enter") );
+			m_contextMenuLink->addAction( "&Open this link in a new tab", ::mainWindow, SLOT(onOpenPageInNewTab()), QKeySequence("Shift+Enter") );
 			
-			m_contextMenuLink->insertItem ( "&Open this link in a new background tab", ::mainWindow, SLOT(slotOpenPageInNewBackgroundTab()), QKeySequence("Ctrl+Enter") );
+			m_contextMenuLink->addAction( "&Open this link in a new background tab", ::mainWindow, SLOT(onOpenPageInNewBackgroundTab()), QKeySequence("Ctrl+Enter") );
 		}
 		
 		setTabKeeper( link );
@@ -251,7 +256,7 @@ void KCHMViewWindow::addNavigationHistory( const QString & url, int scrollpos )
 {
 	// shred the 'forward' history
 	if ( m_historyCurrentPos < m_history.size() )
-		m_history.erase( m_history.at( m_historyCurrentPos ), m_history.end());
+		m_history.erase( m_history.begin() + m_historyCurrentPos, m_history.end() );
 
 	// do not grow the history if already max size
 	if ( m_history.size() >= m_historyMaxSize )
@@ -274,28 +279,11 @@ void KCHMViewWindow::updateNavigationToolbar( )
 	
 	if ( mainWindow )
 	{
-		mainWindow->navigationToolbar()->updateIconStatus( 
-					m_historyCurrentPos > 0,
-					m_historyCurrentPos < (m_history.size() - 1) );
+		mainWindow->navSetBackEnabled( m_historyCurrentPos > 0 );
+		mainWindow->navSetForwardEnabled( m_historyCurrentPos < (m_history.size() - 1) );
 	}
 }
 
-KQPopupMenu * KCHMViewWindow::createListItemContextMenu( QWidget * w )
-{
-	KQPopupMenu * contextMenu = new KQPopupMenu( w );
-		
-	contextMenu->insertItem ( "&Open this link in a new tab",
-							  ::mainWindow, 
-							  SLOT( slotOpenPageInNewTab() ), 
-							  QKeySequence( "Shift+Enter" ) );
-			
-	contextMenu->insertItem ( "&Open this link in a new background tab", 
-							  ::mainWindow, 
-							  SLOT( slotOpenPageInNewBackgroundTab() ),
-							  QKeySequence( "Ctrl+Enter" ) );
-
-	return contextMenu;
-}
 
 void KCHMViewWindow::setTabKeeper( const QString & link )
 {
@@ -305,7 +293,7 @@ void KCHMViewWindow::setTabKeeper( const QString & link )
 	if ( m_newTabLinkKeeper[0] == '#' && !m_openedPage.isEmpty() )
 	{
 			// Clean up opened page URL
-		int pos = m_openedPage.find ('#');
+		int pos = m_openedPage.indexOf('#');
 		QString fixedpath = pos == -1 ? m_openedPage : m_openedPage.left (pos);
 		m_newTabLinkKeeper = fixedpath + m_newTabLinkKeeper;
 	}
